# -*- coding: utf-8 -*-
# @Author  : zhousf-a
# @Function: 业务日志记录
import time
from pathlib import Path
from prettytable import PrettyTable


class Logger(object):
    def __init__(self, log_dir: Path = None, g=None):
        """
        数据链
        :param log_dir: 日志目录，默认空
        :param g: flask.g 默认空
        :param print: 打印日志
        """
        self.log_dir = log_dir
        day_time = time.strftime('%Y/%m/%d %H:%M:%S', time.localtime(time.time())) + '\n'
        if g is not None:
            if hasattr(g, "log"):
                day_time = g.log
            g.logger = self
        # 日志信息-详细
        self.log_txt = day_time
        self.log(log_dir)
        # 日志信息-仅标题
        self.__log_txt_level = []
        # 耗时
        self.elapsed_time_dict = {}

    def elapsed_time(self, k: str, start: float, end: float):
        if k in self.elapsed_time_dict:
            self.elapsed_time_dict[k] += abs(end - start)
        else:
            self.elapsed_time_dict[k] = abs(end - start)

    def print_log(self):
        print(self.log_txt)
        if len(self.elapsed_time_dict) > 0:
            table = PrettyTable()
            table.field_names = self.elapsed_time_dict.keys()
            table.add_row([self.elapsed_time_dict.get(i) for i in self.elapsed_time_dict.keys()])
            print(table)

    @property
    def level_log_first(self):
        txt = [title for (level, title) in self.__log_txt_level if level <= 1]
        return "\n".join(txt)

    @property
    def level_log_second(self):
        txt = [title for (level, title) in self.__log_txt_level if level <= 2]
        return "\n".join(txt)

    @property
    def level_log_third(self):
        txt = [title for (level, title) in self.__log_txt_level if level <= 3]
        return "\n".join(txt)

    def title_first(self, title):
        title = "------------ {0} ------------".format(title)
        self.log_txt = '{0}{1}\n'.format(self.log_txt, title)
        self.__log_txt_level.append((1, title))
        return self

    def title_second(self, title):
        title = "****** {0} ******".format(title)
        self.log_txt = '{0}{1}\n'.format(self.log_txt, title)
        self.__log_txt_level.append((2, title))
        return self

    def title_third(self, title):
        title = "【 {0} 】".format(title)
        self.log_txt = '{0}{1}\n'.format(self.log_txt, title)
        self.__log_txt_level.append((3, title))
        return self

    def log(self, msg):
        """
        记录日志
        :param msg: 信息
        :return:
        """
        if msg is not None:
            self.log_txt = '{0}{1}\n'.format(self.log_txt, msg)
        return self

    # noinspection PyBroadException
    def save_log(self):
        """
        保存日志文件
        :return:
        """
        if self.log_dir is None:
            return
        log_file = "{0}/log.txt".format(self.log_dir)
        log_file = log_file.replace("\\", "/")
        day_time = time.strftime('%Y/%m/%d %H:%M:%S', time.localtime(time.time())) + '\n'
        self.log_txt = '{0}\n{1}'.format(self.log_txt, day_time)
        try:
            with open(log_file, "a+", encoding="utf-8") as f:
                f.write(self.log_txt)
        except Exception as e:
            print(e)
            pass



from distutils.core import setup
from Cython.Build import cythonize

# pip3 --default-timeout=1000 install -U cython
# sudo apt-get  build-dep  gcc


# python3.6 py_to_so.py build_ext --inplace

files = ['command.py']
for f in files:
    setup(ext_modules=cythonize([f]))




# -*- coding:utf-8 -*-
# Author:  zhousf
# Description:
import json
import random
import colorsys
from pathlib import Path
from PIL import Image, ImageDraw, ImageFont
from zhousflib.font import Font_SimSun
FONT = ImageFont.truetype(font=str(Font_SimSun), size=15)


def coco_convert_bbox(box_coco: list):
    _x, _y, _width, _height = tuple(box_coco)
    _bbox = (_x, _y, _x + _width, _y + _height)
    return _bbox


def vis_box_coco(coco_dir: Path, dst_dir: Path):
    """
    可视化coco bbox
    :param coco_dir:
    :param dst_dir:
    :return:
    """
    if not dst_dir.exists():
        dst_dir.mkdir()
    label_json = coco_dir.joinpath("result.json")
    with label_json.open("r", encoding="utf-8") as f:
        result_json = json.loads(f.read())
    # 图片
    _images = result_json["images"]
    # 标签类别
    _categories = result_json["categories"]
    classes_dict = {}
    for item in _categories:
        classes_dict[item.get("id")] = item.get("name")
    # 标注
    _annotations = result_json["annotations"]
    # info
    _info = result_json["info"]
    # 遍历标签
    annotations = {}
    for anno in _annotations:
        image_id = anno["image_id"]
        if image_id not in annotations:
            annotations[image_id] = [anno]
        else:
            annotations[image_id].append(anno)
        assert anno["category_id"] is not None
    img_dir = coco_dir.joinpath("images")
    for img in _images:
        img_file = img_dir.joinpath(img["file_name"])
        bboxes = []
        print(img["id"])
        ann_list = annotations.get(img["id"])
        if not ann_list:
            continue
        for anno in ann_list:
            box = anno.get("bbox")
            class_id = anno.get("category_id")
            bbox_coco = coco_convert_bbox(box)
            # class_id, score, x_min, y_min, x_max, y_max, class_id
            bboxes.append([class_id, "-", bbox_coco[0], bbox_coco[1], bbox_coco[2], bbox_coco[3]])
        image = draw_bbox_label(img_file=img_file, bboxes=bboxes, classes_dict=classes_dict, show=False)
        image.save(dst_dir.joinpath(img_file.name))
        # break


def draw_bbox_label(img_file: Path, bboxes: list, classes_dict, show=False):
    """
    绘制bbox，适用于标注数据，不支持预测可视化
    :param img_file:
    :param bboxes: [[class_id, score, x_min, y_min, x_max, y_max]]
    :param classes_dict: {id1:name1, id2:name2} or [name1, name2]
    :param show:
    :return:
    """
    classes_index = [cls for cls in classes_dict]
    hsv_tuples = [(1.0 * x / len(classes_index), 1., 1.) for x in range(len(classes_index))]
    colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))
    random.seed(0)
    random.shuffle(colors)
    image = Image.open(img_file)
    print(img_file)
    if image.mode != "RGB":
        image = image.convert('RGB')
    image_w, image_h = image.size
    draw = ImageDraw.ImageDraw(image)
    for box in bboxes:
        offset = 0
        class_id = box[-6]
        score = box[-5]
        x_min = int(box[-4])
        y_min = int(box[-3])
        x_max = int(box[-2])
        y_max = int(box[-1])
        if str(class_id).isdigit():
            class_name = classes_dict[class_id]
        else:
            class_name = str(class_id)
        if score != "-":
            class_name += ":{0:.3f}".format(score)
        else:
            class_name += ":{0}".format(score)
        bbox_color = colors[classes_index.index(class_id)]
        width = abs(x_max - x_min)
        height = abs(y_max - y_min)
        draw.rectangle(xy=(x_min, y_min, x_min + width + offset, y_min + height + offset),
                       fill=None, outline=bbox_color, width=1)
        fw, fh = FONT.getsize(class_name)
        if y_min < fh:
            y_min = y_min + fh
        if (x_min + fw) > image_w:
            x_min = x_max - fw
        # draw.rectangle([x_min, y_min, x_min + fw, y_min], fill=(128, 128, 128, 128))
        draw.text(xy=(x_min, y_min - fh), text=class_name, fill=bbox_color, font=FONT)
    if show:
        image.show()
    return image


def draw_bbox_predict(img_file, bboxes: list, classes_dict, score_threshold=0.5, show=False):
    """
    绘制bbox，适用于预测可视化
    :param img_file:
    :param bboxes: [[class_id, score, x_min, y_min, x_max, y_max]]
    :param classes_dict: {id1:name1, id2:name2} or [name1, name2]
    :param score_threshold:
    :param show:
    :return:
    """
    classes_index = [i for i, cls in enumerate(classes_dict)]
    hsv_tuples = [(1.0 * x / len(classes_index), 1., 1.) for x in range(len(classes_index))]
    colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))
    random.seed(0)
    random.shuffle(colors)
    if isinstance(img_file, Path):
        image = Image.open(img_file)
    else:
        image = Image.fromarray(img_file)
    if image.mode != "RGB":
        image = image.convert('RGB')
    image_h, image_w = image.size
    draw = ImageDraw.ImageDraw(image)
    for box in bboxes:
        offset = 0
        class_id = box[-6]
        score = box[-5]
        if score < score_threshold:
            continue
        x_min = int(box[-4])
        y_min = int(box[-3])
        x_max = int(box[-2])
        y_max = int(box[-1])
        if str(class_id).isdigit():
            class_name = classes_dict[class_id]
        else:
            class_name = class_id
            if class_id in classes_dict and isinstance(classes_dict, list):
                class_id = classes_dict.index(class_id)
        class_name += ":{0:.3f}".format(score)
        bbox_color = colors[classes_index.index(class_id)]
        width = abs(x_max - x_min)
        height = abs(y_max - y_min)
        draw.rectangle(xy=(x_min, y_min, x_min + width + offset, y_min + height + offset),
                       fill=None, outline=bbox_color, width=1)
        fw, fh = FONT.getsize(class_name)
        if y_min < fh:
            y_min = y_min + fh
        if (x_min + fw) > image_w:
            x_min = x_max - fw
        # draw.rectangle([x_min, y_min, x_min + fw, y_min], fill=(128, 128, 128, 128))
        draw.text(xy=(x_min, y_min - fh), text=class_name, fill=bbox_color, font=FONT)
    if show:
        image.show()
    return image


if __name__ == "__main__":
    pass



# -*- coding:utf-8 -*-
# Author:      zhousf
# Description: 将整个python工程加密成so
# 1. 安装依赖库
#     pip3 --default-timeout=1000 install -U cython
#     sudo apt-get  build-dep  gcc
# 2. 工程的所有py文件的当前目录以及所有上级目录下都要有__init__.py文件，若没有请新建
# 3. 在工程根目录下或非工程目录外新建build_so目录并将project_to_so.py复制到build_so目录下
# 4. 终端中运行 python3.6 encrypt_project.py build_ext --inplace
# 5. build_so目录下会生成工程所有的so文件和资源文件

import os
import shutil
from pathlib import Path
from distutils.core import setup
from Cython.Build import cythonize

# 工程根目录
project_dir = Path(__file__).parent.parent
# 过滤目录或文件-包含的文件目录下文件不生成so
exclude_dirs_or_files = [
    "{}/venv".format(project_dir),
    "{}/.idea".format(project_dir),
    "{}/.svn".format(project_dir),
    "{}/download".format(project_dir),
    "{}/log".format(project_dir),
    "{}/pid.txt".format(project_dir),
    "{}/app.py".format(project_dir),
    "{}/config.py".format(project_dir),
    "{}/entry.py".format(project_dir),
    "{}/multi_app.py".format(project_dir),
    ]


def copy_file(project_name, file_dir, root, current_file):
    _, child_dir = root.split(project_name)
    if len(child_dir) > 0:
        target_dir = file_dir + "/" + project_name + child_dir
    else:
        target_dir = file_dir + "/" + project_name
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)
    shutil.copy(current_file, target_dir)


def distill_dirs_or_files(root):
    for exclude in exclude_dirs_or_files:
        if root.find(exclude) >= 0:
            return True
    return False


def main():
    project_name = os.path.basename(project_dir)
    file_dir = os.getcwd()
    build_dir = file_dir + "/build"
    # noinspection PyBroadException
    try:
        for root, dirs, files in os.walk(project_dir):
            for file in files:
                current_file = os.path.join(root, file)
                # 过滤py编译文件
                if file.endswith(".pyc"):
                    continue
                if file.endswith(".c"):
                    continue
                # 过滤当前文件
                if current_file == __file__:
                    continue
                # 过滤build文件夹
                if root.find(build_dir) >= 0:
                    continue
                # 过滤build_so文件夹
                if root.find(file_dir) >= 0:
                    continue
                # 过滤指定目录
                if distill_dirs_or_files(root):
                    continue
                # 过滤指定文件
                if current_file in exclude_dirs_or_files:
                    continue
                # 非py文件进行复制操作
                if not file.endswith(".py"):
                    copy_file(project_name, file_dir, root, current_file)
                    continue
                setup(ext_modules=cythonize([current_file]))
                name, _ = file.split(".")
                # 删除.c文件以保证每次都进行so文件生成
                c_file = os.path.join(root, name + ".c")
                if os.path.exists(c_file):
                    os.remove(c_file)
        if os.path.exists(build_dir):
            shutil.rmtree(build_dir)
        print("done! Generating SO files completed.")
        print("SO dir: " + file_dir)
    except Exception:
        if os.path.exists(build_dir):
            shutil.rmtree(build_dir)
        print("工程的所有py文件的当前目录以及所有上级目录中都要有__init__.py文件，若没有请新建！")


main()



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: labelme标注数据转uiex数据集
# prompt标注原则： prompt_tag|word
# 其他标注： 可见即可得
import os
import json
import numpy
import base64
import requests
import numpy as np
from io import BytesIO
from pathlib import Path
from PIL import Image, ImageOps
from zhousflib.util import pil_util


def show_normalize_box(image_file: Path, bbox: list, normalize_size: list, fill_transparent=128):
    """
    显示标准化后的box
    :param image_file:
    :param bbox:
    :param normalize_size: 标准化尺寸
    :param fill_transparent: 填充色透明度[0, 255]，当为-1时则不填充
    :return:
    boxes = [[557, 102, 693, 367],
             [557, 102, 693, 367],
             [557, 102, 693, 367],
             [557, 102, 693, 367],
             [575, 404, 666, 698],
             [575, 404, 666, 698],
             [575, 404, 666, 698],
             [903, 367, 990, 794],
             [346, 529, 431, 794],
             [346, 529, 431, 794],
             [346, 529, 431, 794]]
    show_normalize_box(bbox=boxes, image_file=None, normalize_size=[1000, 1000])
    """
    if normalize_size is None:
        normalize_size = [1000, 1000]
    n_box = []
    image_size = pil_util.get_w_h(image_file=image_file)
    for box in bbox:
        n_box.append(_denormalize_box(box=box, image_size=image_size, normalize_size=normalize_size))
    pil_util.draw_rectangle(bbox=n_box, image_file=image_file, fill_transparent=fill_transparent)


def _normalize_box(box, image_size, normalize_size, offset_x=0, offset_y=0):
    """
    box标准化
    :param box: (x_min, y_min, x_max, y_max)
    :param image_size: [img_w, img_h] 图片宽高
    :param normalize_size: [1000, 1000]
    :param offset_x:
    :param offset_y:
    :return:
    """
    if normalize_size is None:
        normalize_size = [1000, 1000]
    return [
        int((box[0] + offset_x) * normalize_size[0] / image_size[0]),
        int((box[1] + offset_y) * normalize_size[1] / image_size[1]),
        int((box[2] + offset_x) * normalize_size[0] / image_size[0]),
        int((box[3] + offset_y) * normalize_size[1] / image_size[1]),
    ]


def _denormalize_box(box, image_size, normalize_size, offset_x=0, offset_y=0):
    """
    box反标准化
    :param box: (x_min, y_min, x_max, y_max)
    :param image_size: [img_w, img_h] 图片宽高
    :param normalize_size: [1000, 1000]
    :param offset_x:
    :param offset_y:
    :return:
    """
    return [
        int((box[0] - offset_x) * image_size[0] / normalize_size[0]),
        int((box[1] - offset_y) * image_size[1] / normalize_size[0]),
        int((box[2] - offset_x) * image_size[0] / normalize_size[0]),
        int((box[3] - offset_y) * image_size[1] / normalize_size[0])
    ]


def _np2base64(image_np):
    img = Image.fromarray(image_np)
    base64_str = _pil2base64(img)
    return base64_str


def _pil2base64(image, image_type=None, size=False):
    if not image_type:
        image_type = "JPEG"
    img_buffer = BytesIO()
    image.save(img_buffer, format=image_type)

    byte_data = img_buffer.getvalue()
    base64_str = base64.b64encode(byte_data)

    base64_string = base64_str.decode("utf-8")

    if size:
        return base64_string, image.size
    else:
        return base64_string


def _get_buffer(data, file_like=False):
    buff = None
    if len(data) < 1024:
        if os.path.exists(data):
            buff = open(data, "rb").read()
        elif data.startswith("http://") or data.startswith("https://"):
            resp = requests.get(data, stream=True)
            if not resp.ok:
                raise RuntimeError("Failed to download the file from {}".format(data))
            buff = resp.raw.read()
        else:
            raise FileNotFoundError("Image file {} not found!".format(data))
    if buff is None:
        buff = base64.b64decode(data)
    if buff and file_like:
        return BytesIO(buff)
    return buff


def _read_image(image):
    """
    read image to np.ndarray
    """
    image_buff = _get_buffer(image)
    # Use exif_transpose to correct orientation
    _image = np.array(ImageOps.exif_transpose(Image.open(BytesIO(image_buff)).convert("RGB")))
    return _image


def _save_data(save_file: Path, dataset: list):
    with save_file.open("w", encoding="utf-8") as f:
        for item in dataset:
            f.write(json.dumps(item, ensure_ascii=False) + "\n")


def _create_prompt(dataset: list, label_split: str, label_type: dict):
    label_data = []
    for json_file in dataset:
        prompt_list = []
        bbox_list = []
        content = ""
        with json_file.open("r", encoding="utf-8") as f:
            print(json_file)
            data = json.load(f)
            image_file = json_file.parent.joinpath(data["imagePath"])
            if not image_file.exists():
                continue
            image = _read_image(str(image_file))
            image_base64 = _np2base64(image)
            image_width = data["imageWidth"]
            image_height = data["imageHeight"]
            for shape in data["shapes"]:
                label = shape["label"]
                points = shape["points"]
                # shape_type 支持 rectangle 和 polygon
                arr = numpy.asarray(points)
                x_min = numpy.min(arr[:, 0])
                x_max = numpy.max(arr[:, 0])
                y_min = numpy.min(arr[:, 1])
                y_max = numpy.max(arr[:, 1])
                # b_width = abs(x_max - x_min)
                # b_height = abs(y_max - y_min)
                bbox = _normalize_box(box=(x_min, y_min, x_max, y_max), image_size=[image_width, image_height], normalize_size=[1000, 1000])
                if str(label).find(label_split) > -1:
                    prompt = str(label).split(label_split)[0]
                    word = str(label).split(label_split)[-1]
                    start = len(content)
                    content += word
                    end = start + len(word) - 1
                    prompt = label_type.get(prompt, None)
                    if not prompt:
                        continue
                    prompt_list.append((prompt, word, start, end, bbox))
                    for i in range(0, len(word)):
                        bbox_list.append(bbox)
                else:
                    for i in range(0, len(label)):
                        bbox_list.append(bbox)
            if len(prompt_list) > 0:
                for item in prompt_list:
                    prompt, word, start, end, bbox = item
                    prompt_item = {
                        "content": content,
                        "result_list": [
                            {
                                "text": str(word),
                                "start": start,
                                "end": end,
                            }
                        ],
                        "prompt": prompt,
                        "bbox": bbox_list,
                        "image": image_base64,
                    }
                    label_data.append(prompt_item)
    return label_data


def train_test_split(data_dirs: list, dst_dir: Path, label_split: str, label_type: dict, val_size=0.2, test_size=0.2, shuffle=True):
    """
    数据集制作
    :param data_dirs: labelme目录
    :param dst_dir: 保存目录
    :param label_split: prompt标注时的分割符号，可以用"|"
    :param label_type: prompt标注时的标识
    :param val_size: 验证集占比
    :param test_size: 测试集占比
    :param shuffle: 是否打乱顺序
    :return:
    """
    if not dst_dir.exists():
        dst_dir.mkdir(parents=True)
    label_file_list = []
    for labelme_dir in data_dirs:
        for json_file in labelme_dir.rglob("*.json"):
            with json_file.open("r", encoding="utf-8") as f:
                data = json.load(f)
                image_file = json_file.parent.joinpath(data["imagePath"])
                if not image_file.exists():
                    continue
                label_file_list.append(json_file)
    # 打乱顺序
    if shuffle:
        state = np.random.get_state()
        np.random.shuffle(label_file_list)
        np.random.set_state(state)
    # 开始数据集划分
    dataset_val = []
    dataset_test = []
    split_index = 0
    if 1 > val_size > 0:
        split_index = int(len(label_file_list) * val_size)
        dataset_val = label_file_list[:split_index]
    if 1 > test_size > 0:
        start = split_index
        split_index += int(len(label_file_list) * test_size)
        dataset_test = label_file_list[start:split_index]
    dataset_train = label_file_list[split_index:]

    # 训练集
    if len(dataset_train) > 0:
        label_data = _create_prompt(dataset=dataset_train, label_split=label_split, label_type=label_type)
        _save_data(save_file=dst_dir.joinpath("train.txt"), dataset=label_data)
    # 验证集
    if len(dataset_val) > 0:
        label_data = _create_prompt(dataset=dataset_val, label_split=label_split, label_type=label_type)
        _save_data(save_file=dst_dir.joinpath("dev.txt"), dataset=label_data)
    # 测试集
    if len(dataset_test) > 0:
        label_data = _create_prompt(dataset=dataset_test, label_split=label_split, label_type=label_type)
        _save_data(save_file=dst_dir.joinpath("test.txt"), dataset=label_data)
    txt = "train: {0}, val: {1}, test: {2}, total: {3}".format(len(dataset_train), len(dataset_val),
                                                               len(dataset_test), len(label_file_list))
    print(txt)
    readme_txt = dst_dir.joinpath("readme.txt")
    with readme_txt.open("w") as f:
        f.write(txt)


if __name__ == "__main__":
    train_test_split(data_dirs=[Path(r"C:\Users\zhousf-a\Desktop\数据 JSON")],
                     dst_dir=Path(r"C:\Users\zhousf-a\Desktop\uiex"),
                     label_split="|",
                     label_type={"bh": "编号", "gs": "根数", "dj": "等级", "zj": "直径", "dgc": "单根长", "bc": "边长"},
                     val_size=0.2,
                     test_size=0.2,
                     shuffle=True)







# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import numpy
import colorsys
from pathlib import Path
from PIL import Image, ImageDraw


def four_point_convert_bbox(four_points: list):
    """
    四点转换成bbox
    :param four_points: [[252, 140], [300, 140], [300, 189], [252, 189]]
    :return:
    """
    arr = numpy.asarray(four_points)
    x_min = min(arr[:, 0])
    y_min = min(arr[:, 1])
    x_max = max(arr[:, 0])
    y_max = max(arr[:, 1])
    return x_min, y_min, x_max, y_max


def draw_rectangle(bbox: list, image_file: Path = None, image_size: list = None, fill_transparent=255, show=True):
    """
    绘制矩形框
    :param bbox: [(x_min, y_min, x_max, y_max)]
    :param image_file: 空时以空白为背景进行绘制
    :param image_size:
    :param fill_transparent: 填充色透明度[0, 255]，当为-1时则不填充
    :param show:
    :return:
    """
    draw_p = []
    for box in bbox:
        x_min, y_min, x_max, y_max = box
        draw_p.append([(x_min, y_min), (x_max, y_min), (x_max, y_max), (x_min, y_max)])
    return draw_polygon(polygon=draw_p, image_file=image_file, image_size=image_size, fill_transparent=fill_transparent, show=show)


def get_w_h(image_file: Path = None):
    """
    获取图片宽高
    :param image_file:
    :return:
    """
    image = Image.open(image_file)
    return [image.width, image.height]


def draw_polygon(polygon: list, image_file: Path = None, image_size: list = None, fill_transparent=255, show=True):
    """
    绘制四边形
    :param polygon: [[[255, 376], [291, 409], [255, 443], [218, 409]], [[252, 140], [300, 140], [300, 189], [252, 189]]]
    :param image_file: 空时以空白为背景进行绘制
    :param image_size:
    :param fill_transparent: 填充色透明度[0, 255]，当为-1时则不填充
    :param show:
    :return:
    """
    hsv_tuples = [(1.0 * x / len(polygon), 1., 1.) for x in range(len(polygon))]
    colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))
    image_white = None
    if image_size is None:
        image_size = [500, 500]
    if image_file is None:
        image = Image.new('RGBA', (image_size[0], image_size[1]), (255, 255, 255))
        draw = ImageDraw.ImageDraw(image)
    else:
        image = Image.open(image_file)
        if image.mode != "RGBA":
            image = image.convert('RGBA')
        image_white = Image.new('RGBA', (image.width, image.height), (255, 255, 255, 0))
        draw = ImageDraw.ImageDraw(image_white)
    for index, point in enumerate(polygon):
        draw_p = [(p[0], p[1]) for p in point]
        # 边框颜色
        polygon_color = colors[index]
        # 填充颜色+透明
        file_color = (polygon_color[0], polygon_color[1], polygon_color[2], fill_transparent) if fill_transparent > -1 else None
        draw.polygon(draw_p, outline=polygon_color, fill=file_color)
        draw.text(xy=(draw_p[0][0]+1, draw_p[0][1]+1), text=str(index))
    if image_white is not None:
        image.paste(Image.alpha_composite(image, image_white))
    if show:
        image.show()
    return image


if __name__ == "__main__":
    # draw_rectangle([(218, 376, 291, 443)])
    draw_polygon([[[255, 376], [291, 409], [255, 443], [218, 409]], [[252, 140], [300, 140], [300, 189], [252, 189]]])
    pass



# -*- coding:utf-8 -*-
# Author:  zhousf
# Description:
import zipfile
from pathlib import Path
import os
import shutil
import random


def unzip_file(zip_file, dst_dir):
    """
    解压文件
    :param zip_file:
    :param dst_dir:
    :return:
    """
    if not zipfile.is_zipfile(zip_file):
        return False, "It is not a zip file."
    try:
        with zipfile.ZipFile(zip_file, 'r') as f:
            for fn in f.namelist():
                # 解决中文乱码
                name = fn.encode('cp437').decode('utf-8', "ignore")
                if name.startswith("__MACOSX"):
                    continue
                extracted_path = Path(f.extract(fn, dst_dir))
                extracted_path.rename(os.path.join(dst_dir, name))
            # 删除临时文件
            for b in os.listdir(dst_dir):
                current_dir = os.path.join(dst_dir, b)
                if not os.path.isdir(current_dir):
                    continue
                if len(os.listdir(current_dir)) == 0:
                    os.removedirs(current_dir)
    except Exception as ex:
        return False, ex
    return True, "Unzip file successful."


def unzip_flat_file(zip_file, dst_dir):
    """
    解压文件，平铺到dst_dir目录下
    :param zip_file:
    :param dst_dir:
    :return:
    """
    tmp_dir = dst_dir + "/tmp"
    unzip_success, tip = unzip_file(zip_file, tmp_dir)
    if not unzip_success:
        return unzip_success, tip
    try:
        for root, dirs, files in os.walk(tmp_dir):
            for f in files:
                current_file = os.path.join(root, f)
                target_file = os.path.join(dst_dir, f)
                os.rename(current_file, target_file)
    except Exception as ex:
        print(ex)
    if os.path.exists(tmp_dir):
        shutil.rmtree(tmp_dir, ignore_errors=True)
    return True, "Unzip file successful."


def unzip_flat_file_rename(zip_file, dst_dir):
    """
    解压文件，重命名文件并平铺到dst_dir目录下(防止同名文件)
    :param zip_file:
    :param dst_dir:
    :return:
    """
    tmp_dir = dst_dir + "/tmp"
    unzip_success, tip = unzip_file(zip_file, tmp_dir)
    if not unzip_success:
        return unzip_success, tip
    try:
        for root, dirs, files in os.walk(tmp_dir):
            for f in files:
                current_file = os.path.join(root, f)
                target_file = os.path.join(dst_dir, f)
                names = f.split(".")
                if len(names) == 2:
                    name = "{0}_{1}.{2}".format(names[0], random.randint(1000, 9999), names[1])
                    target_file = os.path.join(dst_dir, name)
                os.rename(current_file, target_file)
    except Exception as ex:
        print(ex)
    if os.path.exists(tmp_dir):
        shutil.rmtree(tmp_dir, ignore_errors=True)
    return True, "Unzip file successful."


if __name__ == "__main__":
    zip_file = "/Users/zhousf/Desktop/case/银行卡.zip"
    # zip_file = "/Users/zhousf/Desktop/case/归档.zip"
    dst_dir = "/Users/zhousf/Desktop/case/img"
    print(unzip_flat_file_rename(zip_file, dst_dir))


# -*- coding:utf-8 -*-
# Author:  zhousf
# Description: coco 数据集划分
import shutil
import json
from pathlib import Path
import numpy as np
from prettytable import PrettyTable


def train_test_split(data_dirs: list, dst_dir: Path, val_size=0.2, test_size=0.2, shuffle=True):
    """
    训练集、验证集、测试集划分
    :param data_dirs: 数据集目录
    :param dst_dir: 生成训练集、测试集的目录
    :param val_size: 验证集占比
    :param test_size: 测试集占比
    :param shuffle: 打乱数据集顺序
    :return:
    """
    def split_data(dir_save: Path, dataset: list, annotations: dict):
        image_dir = dir_save.joinpath("images")
        if not image_dir.exists():
            image_dir.mkdir(parents=True)
        annotations_save = []
        for img_d in dataset:
            if img_d["id"] not in image_files:
                continue
            shutil.copy(image_files.get(img_d["id"]), image_dir)
            ann = annotations.get(img_d["id"])
            if ann:
                annotations_save.extend(ann)
        json_file = dir_save.joinpath("result.json")
        data_json = {
            "images": dataset,
            "categories": categories,
            "annotations": annotations_save,
            "info": info,
        }
        with json_file.open("w", encoding="utf-8") as f:
            json.dump(data_json, f, ensure_ascii=False, indent=4)

    images = []
    categories = []
    annotations_dict = {}
    info = {}
    image_files = {}
    img_id = 0
    for data_dir in data_dirs:
        if not data_dir.is_dir():
            continue
        for label_json in data_dir.rglob("*.json"):
            with label_json.open("r", encoding="utf-8") as f:
                result_json = json.loads(f.read())
            # 图片
            _images = result_json["images"]
            # 标签类别
            _categories = result_json["categories"]
            # 标注
            _annotations = result_json["annotations"]
            # info
            _info = result_json["info"]
            tmp_anno = {}
            # 遍历标签
            for anno in _annotations:
                image_id = anno["image_id"]
                if image_id not in tmp_anno:
                    tmp_anno[image_id] = [anno]
                else:
                    tmp_anno[image_id].append(anno)
                assert anno["category_id"] is not None
            # 遍历图片
            for img in _images:
                if img["width"] is None or img["height"] is None:
                    continue
                file_name = img["file_name"]
                print(file_name)
                id_tmp = img["id"]
                file_name = str(file_name).rsplit("/")[-1]
                img["file_name"] = file_name
                img["id"] = img_id
                img_file = data_dir.joinpath("images").joinpath(file_name)
                if not img_file.exists():
                    continue
                image_files[img_id] = img_file
                images.append(img)
                if id_tmp in tmp_anno:
                    ann_ = []
                    for anno in tmp_anno.get(id_tmp):
                        anno["image_id"] = img_id
                        ann_.append(anno)
                    annotations_dict[img_id] = ann_
                img_id += 1
            if len(info) == 0:
                info = _info
            if len(categories) == 0:
                categories = _categories
    # 打乱顺序
    if shuffle:
        state = np.random.get_state()
        np.random.shuffle(images)
        np.random.set_state(state)
    # 开始数据集划分
    dataset_val = []
    dataset_test = []
    split_index = 0
    if 1 > val_size > 0:
        split_index = int(len(images) * val_size)
        dataset_val = images[:split_index]
    if 1 > test_size > 0:
        start = split_index
        split_index += int(len(images) * test_size)
        dataset_test = images[start:split_index]
    dataset_train = images[split_index:]
    # 训练集
    if len(dataset_train) > 0:
        split_data(dir_save=dst_dir.joinpath("train"), dataset=dataset_train, annotations=annotations_dict)
    # 验证集
    if len(dataset_val) > 0:
        split_data(dir_save=dst_dir.joinpath("val"), dataset=dataset_val, annotations=annotations_dict)
    # 测试集
    if len(dataset_test) > 0:
        split_data(dir_save=dst_dir.joinpath("test"), dataset=dataset_test, annotations=annotations_dict)

    txt = "train: {0}, val: {1}, test: {2}, total: {3}".format(len(dataset_train), len(dataset_val),
                                                               len(dataset_test), len(images))
    print(txt)
    readme_txt = dst_dir.joinpath("readme.txt")
    with readme_txt.open("w") as f:
        f.write(txt)


def data_statistics(result_jsons: list):
    """
    数据分布统计
    :param result_jsons: 数据集目录
    :return:
    """
    image_total = 0
    label_list = {}
    for label_json in result_jsons:
        with label_json.open("r", encoding="utf-8") as f:
            result_json = json.loads(f.read())
        # 标签类别
        label_list = result_json["categories"]
        break
    if len(label_list) == 0:
        print("标签类别无效")
        return
    label_names = {k.get("id"): k.get("name") for k in label_list}
    statistics_total = {k.get("id"): 0 for k in label_list}
    for label_json in result_jsons:
        with label_json.open("r", encoding="utf-8") as f:
            result_json = json.loads(f.read())
        # 图片
        image_total += len(result_json["images"])
        # 标注
        for label in result_json["annotations"]:
            label_id = label.get("category_id")
            statistics_total[label_id] += 1
    statistics_total = {label_names.get(k): statistics_total.get(k) for k in statistics_total}
    print(statistics_total)
    table = PrettyTable(field_names=["label", "count"])
    for key in statistics_total:
        table.add_row([key, statistics_total.get(key)])
    print(table)


if __name__ == "__main__":
    pass






# -*- coding:utf-8 -*-
# Author:  zhousf
# Description:
import re


def is_number(string: str):
    """
    是否数值，包含int、float、double
    :param string:
    :return:
    """
    try:
        float(string)
        return True
    except ValueError:
        return False


def is_greater_than_number(string1: str, string2: str):
    """
    string1数值大于string2数值
    :param string1: "12"
    :param string2: "12.0"
    :return:
    """
    if is_number(string1) and is_number(string2):
        if float(string1) > float(string2):
            return True
    if string1 == string2:
        return True
    return False


def is_less_than_number(string1: str, string2: str):
    """
    string1数值小于string2数值
    :param string1: "12"
    :param string2: "12.0"
    :return:
    """
    if is_number(string1) and is_number(string2):
        if float(string1) < float(string2):
            return True
    if string1 == string2:
        return True
    return False


def is_equal_number(string1: str, string2: str):
    """
    数值是否相等
    :param string1: "12"
    :param string2: "12.0"
    :return:
    """
    if is_number(string1) and is_number(string2):
        if float(string1) == float(string2):
            return True
    if string1 == string2:
        return True
    return False


def contains(string: str, what: list):
    for s in what:
        if string.find(s) > -1:
            return True
    return False


def is_empty(obj):
    str_obj = str(obj)
    if str_obj is None or str_obj == 'None':
        return True
    if str_obj.strip() == '':
        return True
    return False


def is_not_empty(obj):
    str_obj = str(obj)
    if str_obj is None or str_obj == 'None':
        return False
    if str_obj.strip() == '':
        return False
    return True


def contain(obj, what):
    str_obj = str(obj)
    str_what = str(what)
    if is_not_empty(str_obj) and is_not_empty(str_what):
        if str_obj.find(str_what) >= 0:
            return True
    return False


def not_contain(obj, what):
    str_obj = str(obj)
    str_what = str(what)
    if is_not_empty(str_obj) and is_not_empty(str_what):
        if str_obj.find(str_what) >= 0:
            return False
    return True


def right_just(obj, length, fill_char=' '):
    """
    左补齐（右调整）
    :param obj: abc
    :param length: 5
    :param fill_char:
    :return: '  abc'
    """
    if not isinstance(obj, str):
        obj = str(obj)
    return obj.rjust(length, fill_char)


def left_just(obj, length, fill_char=' '):
    """
    右补齐（左调整）
    :param obj: abc
    :param length: 5
    :param fill_char:
    :return: 'abc  '
    """
    if not isinstance(obj, str):
        obj = str(obj)
    return obj.ljust(length, fill_char)


def center_just(obj, length):
    """
    中间补齐（两端调整）
    :param obj: abc
    :param length: 5
    :return: ' abc '
    """
    if not isinstance(obj, str):
        obj = str(obj)
    return obj.center(length)


def only_digit_letter_chinese(string):
    """
    同时包含中文 & (数字 or 字母)
    :param string:
    :return:
    """
    # 提取数字
    match_digit = re.sub(u"([^\u0030-\u0039])", "", string)
    # 提取大小写字母
    match_letter = re.sub(u"([^\u0041-\u005a\u0061-\u007a])", "", string)
    # 提取汉字
    match_chinese = re.sub(u"([^\u4e00-\u9fa5])", "", string)
    if len(match_chinese) == 0:
        return False
    if len(match_digit) + len(match_letter) == 0:
        return False
    return (len(match_chinese) + len(match_digit) + len(match_letter)) == len(string)


def only_digit_letter(string):
    """
    同时包含数字和字母
    :param string:
    :return:
    """
    # 提取数字
    match_digit = re.sub(u"([^\u0030-\u0039])", "", string)
    # 提取大小写字母
    match_letter = re.sub(u"([^\u0041-\u005a\u0061-\u007a])", "", string)
    if len(match_digit) == 0 or len(match_letter) == 0:
        return False
    return (len(match_digit) + len(match_letter)) == len(string)


def digit_or_letter(string):
    """
    包含数字或字母
    :param string:
    :return:
    """
    # 提取数字
    match_digit = re.sub(u"([^\u0030-\u0039])", "", string)
    # 提取大小写字母
    match_letter = re.sub(u"([^\u0041-\u005a\u0061-\u007a])", "", string)
    return (len(match_digit) + len(match_letter)) == len(string)


def only_digit(string):
    """
    只包含数字
    :param string:
    :return:
    """
    return string.isdigit()


def only_letter(string):
    """
    只包含字母
    :param string:
    :return:
    """
    # 提取大小写字母
    match_string = re.sub(u"([^\u0041-\u005a\u0061-\u007a])", "", string)
    return len(match_string) == len(string)